@@ -5,15 +5,16 @@ Configuration program for botan (http://botan.randombit.net/)
   (C) 2009-2011 Jack Lloyd
   Distributed under the terms of the Botan license
 
-Tested with
-   CPython 2.5, 2.6, 2.7 - OK
-   Jython 2.5 - Target detection does not work (use --os and --cpu)
+Tested with CPython 2.6, 2.7, 3.1 and PyPy 1.5
 
-   CPython 2.4 and earlier are not supported
+Python 2.5 works if you change the exception catching syntax:
+   perl -pi -e 's/except (.*) as (.*):/except $1, $2:/g' configure.py
 
-   Use the 2to3 script to use with Python 3
+Jython - Target detection does not work (use --os and --cpu)
 
-   Has not been tested with IronPython or PyPy
+CPython 2.4 and earlier are not supported
+
+Has not been tested with IronPython
 """
 
 import sys
@@ -41,17 +42,30 @@ def flatten(l):
     return sum(l, [])
 
 def get_vc_revision():
-    mtn = subprocess.Popen(['mtn', 'automate', 'heads'],
-                           stdout=subprocess.PIPE,
-                           stderr=subprocess.PIPE)
+    try:
+        mtn = subprocess.Popen(['mtn', 'automate', 'heads'],
+                               stdout=subprocess.PIPE,
+                               stderr=subprocess.PIPE,
+                               universal_newlines=True)
 
-    (stdout, stderr) = mtn.communicate()
+        (stdout, stderr) = mtn.communicate()
 
-    if(stderr != ''):
-        #logging.debug('Error getting rev from monotone - %s' % (stderr))
+        if mtn.returncode != 0:
+            logging.debug('Error getting rev from monotone - %d (%s)'
+                          % (mtn.returncode, stderr))
+            return 'unknown'
+
+        rev = str(stdout).strip()
+        logging.debug('Monotone reported revision %s' % (rev))
+
+        return 'mtn:' + rev
+    except OSError as e:
+        logging.debug('Error getting rev from monotone - %s' % (e[1]))
+        return 'unknown'
+    except Exception as e:
+        logging.debug('Error getting rev from monotone - %s' % (e))
         return 'unknown'
 
-    return 'mtn:' + stdout.strip()
 
 class BuildConfigurationInformation(object):
 
@@ -65,7 +79,7 @@ class BuildConfigurationInformation(object):
 
     version_datestamp = botan_version.release_datestamp
 
-    version_vc_rev = botan_version.release_vc_rev or get_vc_revision()
+    version_vc_rev = botan_version.release_vc_rev
     version_string = '%d.%d.%d' % (version_major, version_minor, version_patch)
 
     """
@@ -73,6 +87,9 @@ class BuildConfigurationInformation(object):
     """
     def __init__(self, options, modules):
 
+        if self.version_vc_rev is None:
+            self.version_vc_rev = get_vc_revision()
+
         self.build_dir = os.path.join(options.with_build_dir, 'build')
 
         self.checkobj_dir = os.path.join(self.build_dir, 'checks')
@@ -501,7 +518,13 @@ def lex_me_harder(infofile, to_obj, allowed_groups, name_val_pairs):
                                      lexer.lineno)
 
         elif token in name_val_pairs.keys():
-            to_obj.__dict__[token] = lexer.get_token()
+            next_val = lexer.get_token()
+
+            if type(to_obj.__dict__[token]) is list:
+                to_obj.__dict__[token].append(next_val)
+            else:
+                to_obj.__dict__[token] = next_val
+
         else: # No match -> error
             raise LexerError('Bad token "%s"' % (token), lexer.lineno)
 
@@ -515,34 +538,36 @@ def force_to_dict(l):
 Represents the information about a particular module
 """
 class ModuleInfo(object):
+
     def __init__(self, infofile):
 
         lex_me_harder(infofile, self,
                       ['source', 'header:internal', 'header:public',
                        'requires', 'os', 'arch', 'cc', 'libs',
-                       'comment'],
+                       'comment', 'warning'],
                       {
                         'load_on': 'auto',
-                        'define': None,
+                        'define': [],
                         'uses_tr1': 'false',
                         'need_isa': None,
                         'mp_bits': 0 })
 
-        if self.source == [] and \
-            self.header_internal == [] and \
-            self.header_public == []:
-
-            for (dirpath, dirnames, filenames) in os.walk(self.lives_in):
-                if dirpath == self.lives_in:
+        def extract_files_matching(basedir, suffixes):
+            for (dirpath, dirnames, filenames) in os.walk(basedir):
+                if dirpath == basedir:
                     for filename in filenames:
                         if filename.startswith('.'):
                             continue
 
-                        if filename.endswith('.cpp') or \
-                           filename.endswith('.S'):
-                            self.source.append(filename)
-                        elif filename.endswith('.h'):
-                            self.header_public.append(filename)
+                        for suffix in suffixes:
+                            if filename.endswith(suffix):
+                                yield filename
+
+        if self.source == []:
+            self.source = list(extract_files_matching(self.lives_in, ['.cpp', '.S']))
+
+        if self.header_internal == [] and self.header_public == []:
+            self.header_public = list(extract_files_matching(self.lives_in, ['.h']))
 
         # Coerce to more useful types
         def convert_lib_list(l):
@@ -579,6 +604,11 @@ class ModuleInfo(object):
         else:
             self.comment = None
 
+        if self.warning != []:
+            self.warning = ' '.join(self.warning)
+        else:
+            self.warning = None
+
     def sources(self):
         return self.source
 
@@ -588,6 +618,9 @@ class ModuleInfo(object):
     def internal_headers(self):
         return self.header_internal
 
+    def defines(self):
+        return ['HAS_' + d for d in self.define]
+
     def compatible_cpu(self, archinfo, options):
 
         arch_name = archinfo.basename
@@ -910,19 +943,30 @@ def fixup_proc_name(proc):
 def canon_processor(archinfo, proc):
     proc = fixup_proc_name(proc)
 
+    # First, try to search for an exact match
     for ainfo in archinfo.values():
         if ainfo.basename == proc or proc in ainfo.aliases:
             return (ainfo.basename, ainfo.basename)
-        else:
-            for (match,submodel) in ainfo.all_submodels():
-                if re.search(match, proc) != None:
-                    return (ainfo.basename, submodel)
+
+        for (match,submodel) in ainfo.all_submodels():
+            if proc == submodel or proc == match:
+                return (ainfo.basename, submodel)
+
+    logging.debug('Could not find an exact match for CPU "%s"' % (proc))
+
+    # Now, try searching via regex match
+    for ainfo in archinfo.values():
+        for (match,submodel) in ainfo.all_submodels():
+            if re.search(match, proc) != None:
+                logging.debug('Possible match "%s" with "%s" (%s)' % (
+                    proc, match, submodel))
+                return (ainfo.basename, submodel)
 
     logging.debug('Known CPU names: ' + ' '.join(
-        sorted(sum([[ainfo.basename] + \
-                    ainfo.aliases + \
-                    [x for (x,_) in ainfo.all_submodels()]
-                    for ainfo in archinfo.values()], []))))
+        sorted(flatten([[ainfo.basename] + \
+                        ainfo.aliases + \
+                        [x for (x,_) in ainfo.all_submodels()]
+                        for ainfo in archinfo.values()]))))
 
     raise Exception('Unknown or unidentifiable processor "%s"' % (proc))
 
@@ -963,7 +1007,7 @@ def process_template(template_file, variables):
     try:
         template = PercentSignTemplate(slurp_file(template_file))
         return template.substitute(variables)
-    except KeyError, e:
+    except KeyError as e:
         raise Exception('Unbound var %s in template %s' % (e, template_file))
 
 """
@@ -1110,8 +1154,7 @@ def create_template_vars(build_config, options, modules, cc, arch, osinfo):
 
         'link_to': ' '.join([cc.add_lib_option + lib for lib in link_to()]),
 
-        'module_defines': make_cpp_macros(
-            sorted(['HAS_' + m.define for m in modules if m.define])),
+        'module_defines': make_cpp_macros(sorted(flatten([m.defines() for m in modules]))),
 
         'target_os_defines': make_cpp_macros(osinfo.defines()),
 
@@ -1283,6 +1326,8 @@ def choose_modules_to_use(modules, archinfo, options):
             logging.info('Using SIMD module ' + mod)
         if modules[mod].comment:
             logging.info('%s: %s' % (mod, modules[mod].comment))
+        if modules[mod].warning:
+            logging.warning('%s: %s' % (mod, modules[mod].warning))
 
     logging.debug('Loading modules %s', ' '.join(sorted(to_load)))
 
@@ -1404,14 +1449,14 @@ def setup_build(build_config, options, template_vars):
     try:
         if options.clean_build_tree:
             shutil.rmtree(build_config.build_dir)
-    except OSError, e:
+    except OSError as e:
         if e.errno != errno.ENOENT:
             logging.error('Problem while removing build dir: %s' % (e))
 
     for dir in build_config.build_dirs:
         try:
             os.makedirs(dir)
-        except OSError, e:
+        except OSError as e:
             if e.errno != errno.EEXIST:
                 logging.error('Error while creating "%s": %s' % (dir, e))
 
@@ -1462,7 +1507,7 @@ def setup_build(build_config, options, template_vars):
         for header_file in header_list:
             try:
                 portable_symlink(header_file, dir, link_method)
-            except OSError, e:
+            except OSError as e:
                 if e.errno != errno.EEXIST:
                     logging.error('Error linking %s into %s: %s' % (
                         header_file, dir, e))
@@ -1634,6 +1679,9 @@ def main(argv = None):
     if argv is None:
         argv = sys.argv
 
+    logging.basicConfig(stream = sys.stdout,
+                        format = '%(levelname) 7s: %(message)s')
+
     options = process_command_line(argv[1:])
 
     def log_level():
@@ -1643,9 +1691,7 @@ def main(argv = None):
             return logging.WARNING
         return logging.INFO
 
-    logging.basicConfig(stream = sys.stdout,
-                        format = '%(levelname) 7s: %(message)s',
-                        level = log_level())
+    logging.getLogger().setLevel(log_level())
 
     logging.debug('%s invoked with options "%s"' % (
         argv[0], ' '.join(argv[1:])))
@@ -1664,15 +1710,6 @@ def main(argv = None):
 
     (modules, archinfo, ccinfo, osinfo) = load_info_files(options)
 
-    if options.os is None:
-        options.os = platform.system().lower()
-
-        if re.match('^cygwin_.*', options.os):
-            logging.debug("Converting '%s' to 'cygwin'", options.os)
-            options.os = 'cygwin'
-
-        logging.info('Guessing target OS is %s (use --os to set)' % (options.os))
-
     if options.compiler is None:
         if options.os == 'windows':
             if have_program('g++') and not have_program('cl'):
@@ -1684,6 +1721,18 @@ def main(argv = None):
         logging.info('Guessing to use compiler %s (use --cc to set)' % (
             options.compiler))
 
+    if options.os is None:
+        options.os = platform.system().lower()
+
+        if re.match('^cygwin_.*', options.os):
+            logging.debug("Converting '%s' to 'cygwin'", options.os)
+            options.os = 'cygwin'
+
+        if options.os == 'windows' and options.compiler == 'gcc':
+            logging.warning('Detected GCC on Windows; use --os=cygwin or --os=mingw?')
+
+        logging.info('Guessing target OS is %s (use --os to set)' % (options.os))
+
     if options.compiler not in ccinfo:
         raise Exception('Unknown compiler "%s"; available options: %s' % (
             options.compiler, ' '.join(sorted(ccinfo.keys()))))
@@ -1723,12 +1772,22 @@ def main(argv = None):
 
         def get_gcc_version(gcc_bin):
             try:
-                gcc_version = ''.join(subprocess.Popen(
+                gcc_proc = subprocess.Popen(
                     gcc_bin.split(' ') + ['-dumpversion'],
                     stdout=subprocess.PIPE,
-                    stderr=subprocess.PIPE).communicate()).strip()
+                    stderr=subprocess.PIPE,
+                    universal_newlines=True)
+
+                (stdout, stderr) = gcc_proc.communicate()
+
+                if gcc_proc.returncode != 0:
+                    logging.warning("GCC returned non-zero result %s" % (stderr))
+                    return None
+
+                gcc_version = stdout.strip()
+
                 logging.info('Detected gcc version %s' % (gcc_version))
-                return gcc_version
+                return [int(v) for v in gcc_version.split('.')]
             except OSError:
                 logging.warning('Could not execute %s for version check' % (gcc_bin))
                 return None
@@ -1740,24 +1799,29 @@ def main(argv = None):
 
         gcc_version = get_gcc_version(options.compiler_binary or cc.binary_name)
 
+        def gcc_version_matches(matches):
+            for match in matches.items():
+                if gcc_version[0] != match[0]:
+                    continue
+
+                for minor in match[1]:
+                    if minor == gcc_version[1]:
+                        return True
+            return False
+
         if gcc_version:
 
             if not is_64bit_arch(options.arch) and not options.dumb_gcc:
-                matching_version = '(4\.[01234]\.)|(3\.[34]\.)|(2\.95\.[0-4])'
-
-                if re.search(matching_version, gcc_version):
+                if gcc_version_matches({ 4 : [0, 1, 2, 3, 4], 3 : [3, 4], 2 : [95] }):
                     options.dumb_gcc = True
 
-            versions_without_tr1 = '(4\.0\.)|(3\.[0-4]\.)|(2\.95\.[0-4])'
-
             if options.with_tr1 == None and \
-               re.search(versions_without_tr1, gcc_version):
+                    gcc_version_matches({ 4 : [0], 3 : [0,1,2,3,4], 2 : [95] }):
                 logging.info('Disabling TR1 support for this gcc, too old')
                 options.with_tr1 = 'none'
 
-            versions_without_visibility = '(3\.[0-4]\.)|(2\.95\.[0-4])'
             if options.with_visibility == None and \
-               re.search(versions_without_visibility, gcc_version):
+                    gcc_version_matches({ 3 : [0,1,2,3,4], 2 : [95] }):
                 logging.info('Disabling DSO visibility support for this gcc, too old')
                 options.with_visibility = False
 
@@ -1821,9 +1885,9 @@ def main(argv = None):
 if __name__ == '__main__':
     try:
         main()
-    except Exception, e:
+    except Exception as e:
         logging.error(str(e))
-        #import traceback
-        #traceback.print_exc(file=sys.stderr)
+        import traceback
+        logging.debug(traceback.format_exc())
         sys.exit(1)
     sys.exit(0)
