@@ -1,12 +1,3 @@
-/*
-Decrypt an encrypted DSA private key. Then use that key to sign a message.
-
-Written by Jack Lloyd (lloyd@randombit.net), August 5, 2002
-   Updated to use X.509 and PKCS #8 format keys, October 21, 2002
-
-This file is in the public domain
-*/
-
 #include <iostream>
 #include <iomanip>
 #include <fstream>
@@ -14,8 +5,9 @@ This file is in the public domain
 #include <memory>
 
 #include <botan/botan.h>
-#include <botan/look_pk.h>
+#include <botan/pubkey.h>
 #include <botan/dsa.h>
+#include <botan/base64.h>
 using namespace Botan;
 
 const std::string SUFFIX = ".sig";
@@ -64,16 +56,14 @@ int main(int argc, char* argv[])
          return 1;
          }
 
-      PK_Signer signer(*dsakey, get_emsa("EMSA1(SHA-1)"));
+      PK_Signer signer(*dsakey, "EMSA1(SHA-1)");
 
       DataSource_Stream in(message);
       byte buf[4096] = { 0 };
-      while(u32bit got = in.read(buf, sizeof(buf)))
+      while(size_t got = in.read(buf, sizeof(buf)))
          signer.update(buf, got);
 
-      Pipe pipe(new Base64_Encoder);
-      pipe.process_msg(signer.signature(rng));
-      sigfile << pipe.read_all_as_string() << std::endl;
+      sigfile << base64_encode(signer.signature(rng)) << "\n";
    }
    catch(std::exception& e)
       {
